#! /bin/sh

# Script assumes that CXX and CXXFLAGS are set.
# This script tests whether CXX can compile with gmp (and gmpxx)
# by just adding -lgmp (and -lgmpxx) as flags.
# The check entails compiling very simple source files.

# Exit code is non-zero if "-lgmp" does not work (or an error occurs).
# Exit code is 0 if "-lgmp" works; output is "GMPXX" if -lgmpxx works,
# otherwise "GMP" if just -lgmp works.

if [ -z "$CXX" ]
then
  echo "$0: ERROR: environment variable CXX not set."
  exit 1
fi

if [ ! -d configuration ]
then
  echo "$0: ERROR: please run in the CoCoALib root directory"
  exit 1
fi

# Now create a temp dir in which to work:
# I use a temp dir so all junk created by the compiler can be removed
# simply by removing the directory -- also I do not need to worry about
# stomping on existing files.

umask 22
TMP_DIR=TMPDIR-gmp-try-default-$HOSTNAME-$UID-$$
/bin/rm -rf $TMP_DIR  &&  mkdir $TMP_DIR  2>/dev/null
if [ $? -ne 0 ]
then
  echo "ERROR: Unable to create working directory ($TMP_DIR)"
  exit 1
fi

cd $TMP_DIR
# define fn for cleaning up temp dir
cleanup()
{
  cd ..
  /bin/rm  -rf "$TMP_DIR"
}


# Here is the simple source code we shall use to test for gmp:
cat > TestGMP.C <<EOF
#include "gmp.h"
#include <iostream>

int main()
{
  mpz_t i;
  mpz_init(i);
  mpz_set_si(i, 12345);
  std::cout << __GNU_MP_VERSION << "." << __GNU_MP_VERSION_MINOR << "." << __GNU_MP_VERSION_PATCHLEVEL << std::endl;
  mpz_clear(i);
}
EOF

# Here is the simple source code we shall use to test for gmpxx:
cat > TestGMPXX.C <<EOF
#include "gmpxx.h"
#include <iostream>

int main()
{
  mpz_class i(12345);
  std::cout << __GNU_MP_VERSION << "." << __GNU_MP_VERSION_MINOR << "." << __GNU_MP_VERSION_PATCHLEVEL << std::endl;
}
EOF

# Try different architectures (in case we're on a 32/64 bit platform)
for ARCHFLAG in "" "-m64" "-m32"
do
  # Try plain GMP
  $CXX $ARCHFLAG $CXXFLAGS TestGMP.C -o TestGMP -lgmp > /dev/null 2>&1
  if [ $? -ne 0 -o \! -f TestGMP -o \! -x TestGMP ]
  then
    # compilation failed so try next ARCHFLAG
    continue;
  fi

  # Default GMP worked, now try GMPXX
  $CXX $ARCHFLAG $CXXFLAGS TestGMPXX.C -o TestGMPXX -lgmpxx -lgmp > /dev/null 2>&1
  if [ $? -ne 0 ]
  then
    # GMPXX test failed, so we have only GMP
    echo GMP
  else
    # GMPXX passed, so we have both GMP and GMPXX
    echo GMPXX
  fi
  cleanup
  exit 0
done

# No default GMP installation found
echo "No default GMP installation (with CXXFLAGS=\"$CXXFLAGS\")"
exit 1
